package com.danitheskunk.skunkworks.audio;

import org.lwjgl.openal.AL;
import org.lwjgl.openal.ALC;

import java.nio.ByteBuffer;
import java.nio.IntBuffer;
import java.util.Random;

import static org.lwjgl.openal.AL10.*;
import static org.lwjgl.openal.ALC10.*;

public class AudioEngine {
	private int sampleRate;
	private int bufferSize;
	private int bufferCount;
	private long device;
	private long context;
	private int source;
	private int tick = 0;

	public AudioEngine(int sampleRate, int bufferSize, int bufferCount) {
		this.sampleRate = sampleRate;
		this.bufferSize = bufferSize;
		this.bufferCount = bufferCount;

		device = alcOpenDevice((ByteBuffer) null);
		if(device == 0) {
			throw new RuntimeException("couldn't init openal device");
		}
		var deviceCaps = ALC.createCapabilities(device);
		context = alcCreateContext(device, (IntBuffer) null);
		if(context == 0) {
			throw new RuntimeException("couldn't create openal context");
		}
		alcMakeContextCurrent(context);
		AL.createCapabilities(deviceCaps);

		this.source = alGenSources();

		for(int i = 0; i < bufferCount; ++i) {
			addEmptyBuffer();
		}
		alSourcePlay(source);
	}

	public void close() {
		alcDestroyContext(context);
		alcCloseDevice(device);
	}

	private void fillBuffer(int buffer) {
		var buf = new double[bufferSize * 2];
		for(int i = 0; i < bufferSize; ++i) {
			double time = (double) tick / (double) sampleRate;
			var s = Math.sin(2 * Math.PI * 440 * time);
			buf[i * 2] = s;
			buf[i * 2 + 1] = s;
			++tick;
		}

		var shortBuffer = new short[bufferSize * 2];
		for(int i = 0; i < bufferSize * 2; ++i) {
			shortBuffer[i] = (short) (buf[i] * 32760);
		}

		alBufferData(buffer, AL_FORMAT_STEREO16, shortBuffer, sampleRate);
	}

	private void addEmptyBuffer() {
		var buf = alGenBuffers();
		fillBuffer(buf);
		alSourceQueueBuffers(source, buf);
	}

	public void refill() {
		var bufCount = alGetSourcei(source, AL_BUFFERS_PROCESSED);
		var bufs = new int[bufCount];
		alSourceUnqueueBuffers(source, bufs);
		for(int i = 0; i < bufCount; ++i) {
			fillBuffer(bufs[i]);
			alSourceQueueBuffers(source, bufs[i]);
		}
		if(alGetSourcei(source, AL_SOURCE_STATE) != AL_PLAYING) {
			System.out.println("buffer underrun, addng buffer");
			addEmptyBuffer();
			alSourcePlay(source);
		}
	}

	public int getSampleRate() {
		return sampleRate;
	}
}
